// AnnoyingKid.java
//
// CompSci 141 / CSE 141 / Informatics 101 Spring 2013
// Code Example
//
// An AnnoyingKid object will be managed by a thread, so it will implement
// the Runnable interface.  This means that the run() method of an AnnoyingKid
// object will make up the "body" of a thread (i.e. the thread will run for as
// long as its AnnoyingKid's run() method runs).
//
// An AnnoyingKid is characterized by three fields:
//
//   * the name of the kid
//   * the thing that the kid wants
//   * the delay, in milliseconds, between the kid asking for what s/he wants
//
// When you call an AnnoyingKid's run() method, it will repeatedly ask for
// what it wants until the thread the run() method is executing on is
// interrupted.  This is an important thing to realize: if you simply call
// run() on an existing thread, the existing thread will be stuck in an
// infinite loop (until some other thread interrupts it).  But if you run
// the AnnoyingKid in a separate thread, you don't have this problem;
// separate threads progress independently.

public class AnnoyingKid implements Runnable
{
	private String name;
	private String wants;
	private int delay;
	

	public AnnoyingKid(String name, String wants, int delay)
	{
		this.name = name;
		this.wants = wants;
		this.delay = delay;
	}


	// The run() method is the body of an annoying kid thread.
	public void run()
	{
		System.out.println(name + " has entered the room.");
		
		try
		{
			// An annoying kid thread uses its interrupted status to know
			// whether it should stop or continue.  The simplest way to
			// implement this behavior is to have the thread run a loop
			// forever, in this case, because each loop iteration calls
			// Thread.sleep().  We know that Thread.sleep() throws an
			// InterruptedException when the thread's interrupted flag
			// is set -- even if the flag is set before the call to
			// Thread.sleep() -- so as long as we catch the exception
			// outside of the loop, that provides a handy mechanism to
			// bail us out of the loop.
			//
			// Of course, you can't always do this; not all threads have
			// loops that call Thread.sleep().  A thread can ask for its
			// own interrupted status by calling the static
			// Thread.interrupted() method.  The method is static because a
			// thread should only be able to ask for its own interrupted
			// status.  So the static Thread.interrupted() method
			// returns the interrupted status for the current thread
			// (i.e., whichever thread executed it).  And, interestingly,
			// asking for the status also has the effect of clearing it
			// automatically.
	
			while (true)
			{
				// The annoying kid will repeatedly sleep for its specified
				// delay, then say what (s)he wants.  This is easily done with
				// the following code...
				//
				// Thread.sleep() is a static method that causes the current
				// thread (i.e., whichever thread executed it) to sleep for the
				// given number of milliseconds.  After that number of
				// milliseconds, the thread will again be in the "runnable"
				// state, so that it may be scheduled to receive processor
				// time.  It's important to note that there may be a delay
				// between the given time running out and the thread actually
				// being given processor time.  When the sleep time has
				// elapsed, the thread becomes eligible to run again, but will
				// not run again until it gets a slice of processor time.
				//
				// Thread.sleep() will immediately terminate with an
				// InterruptedException if the thread is interrupted while
				// sleeping.  This will cause control to be passed to the catch
				// block outside of the loop, effectively terminating the loop.
				//
				// If the interrupted status is true at the time Thread.sleep()
				// is called, Thread.sleep() immediately terminates with an
				// InterruptedException.
	
				Thread.sleep(delay);
				System.out.println(name + " says 'I want " + wants + "!'");
			}
		}
		catch (InterruptedException e)
		{
			// We don't do anything here when we catch the exception, because
			// all we want the thread to do when it is interrupted is die.
			// So we just catch the exception and fall out of the catch block.
		}


		// We're out of the loop!  Print a final message and exit the run()
		// method (which ends the thread).

		System.out.println(name + " has left the room and is no longer annoying us.");
	}
}
