// AnnoyingKidsExample.java
//
// CompSci 141 / CSE 141 / Informatics 101 Spring 2013
// Code Example
//
// This is a commented version of the AnnoyingKid example from lecture,
// which illustrates how to start and interrupt threads.  The threads used
// in this example do not communicate with each other at all, except for
// interruption.  A subsequent code example will demonstrate the use of the
// synchronized keyword, as well as the wait() and notify() methods.

import java.util.Scanner;



// The Main class contains the main() method.  The program will start up four
// annoying kids, each in its own thread, then allow them to repeatedly ask for
// what they want you until you press the Enter key, at which point they will
// stop.
//
// The program will manage the four annoying kids in separate threads.  In
// fact, the program will actually have five threads running:
//
//   * The "main thread" will start the four annoying kids, then wait for
//     the user to press Enter.  When Enter is pressed, the main thread will
//     tell the four annoying kids to stop, wait until they've stopped, then
//     end the program.
//
//   * Each of the annoying kids will be managed by its own thread.  The
//     thread will run a loop that alternately prints a message and sleeps
//     for some number of milliseconds.  This loop will continue until the
//     thread is interrupted, at which time it will cleanly return from its
//     run() method (cleanly killing its thread).  Even if the thread is
//     sleeping at the time it is interrupted, it will immediately wake up
//     and end itself cleanly.

public class Main
{
	// In our program, the main() method is executed by the main thread.  It
	// will start and stop the threads that manage the annoying kids.

	public static void main(String[] args)
	{
		System.out.println("Starting the four annoying kids...");


		// A Thread object represents a thread in a Java program.  Threads each
		// have a job to do; most often, that job is represented by a Runnable
		// object.  (A Runnable object is one that has a run() method; that's
		// the only requirement.)  When the thread starts, its Runnable object's
		// run() method is called.  The thread dies when the run() method
		// returns.  In this case, each Thread object will manage one
		// AnnoyingKid.

		Thread t1 = new Thread(new AnnoyingKid("Alex", "Boo", 1000));
		Thread t2 = new Thread(new AnnoyingKid("Dude", "a clean rug", 7500));
		Thread t3 = new Thread(new AnnoyingKid("Calvin", "an intact cranium", 3200));
		Thread t4 = new Thread(new AnnoyingKid("Derek", "an 'A' in CS 141", 1850));


		// Creating the Thread objects above doesn't actually start the threads
		// executing; it just gets them ready to run.  In order to actually
		// start them, you call the start() method on them.  The start() method
		// fires off the new thread (which calls the run() method on its
		// Runnable) then returns control back to this thread, so that both can
		// execute concurrently.  The four lines below start the four annoying
		// kid threads.

		t1.start();
		t2.start();
		t3.start();
		t4.start();


		// At this point in the program, all four annoying kids will begin
		// their tirades.  Meanwhile, the main thread will sit and wait for
		// user input.


		// I intend to read a line of input from the console (to check whether
		// the user has pressed the Enter key), so I'll need a Scanner wrapped
		// around System.in to do that.

		Scanner in = new Scanner(System.in);
		
		// Now I'll read a line of input.  The beauty of this is that the
		// other threads will continue making progress, even as the main
		// thread sits and waits for user input.  One thread being blocked
		// doesn't cause other threads to be blocked, unless they're all
		// blocked waiting for the same resource (as we'll see in a future
		// example).

		in.nextLine();


		System.out.println("Stopping the four annoying kids...");


		// Now that I've read a line of input (i.e. the user has pressed
		// Enter), it's time to stop the annoying kids.  I'll do so by using
		// interruption, as we talked about in lecture.
		//
		// Recall that every thread has an "interrupted status" associated
		// with it.  The interrupted status is a flag (i.e. true/false).
		// Calling the interrupt() method on some thread sets its interrupted
		// status to true.  A well-behaved thread will pay heed to this
		// status change and stop (or at least interrupt) what it's doing,
		// but it is not required to.  Certain long-running things tasks that
		// block threads from making progress, such as sleeping, will force
		// the interruption to have an effect, e.g., throwing an exception
		// such as an InterruptedException, but the thread can catch the
		// resulting exception and keep right on truckin'.

		t1.interrupt();
		t2.interrupt();
		t3.interrupt();
		t4.interrupt();


		// Since the annoying kid threads should die soon after they're
		// interrupted, but perhaps not immediately afterward, the main
		// thread should wait until all the annoying kid threads have died
		// before printing a final message and ending the program.  This can
		// be done by using the join() method.  A line of code like this:
		//
		//     t1.join();
		//
		// says "The current thread should block until the thread t1 has
		// died."  (Remember that a thread dies when its run() method stops.)
		//
		// If I call join() on four threads in some order (and it doesn't
		// matter which order), I'm basically saying "Wait until all four
		// annoying kid threads have died before continuing."
		//
		// The reason why I have to catch InterruptedException in this case is
		// because join() is another one of those methods that could take a
		// long time (or even forever!) to return.  So that a thread may
		// still be interrupted even while waiting for a join, the join()
		// method can throw an InterruptedException.  Since we know that no
		// one will attempt to interrupt the main thread in this program,
		// we'll just catch and ignore the InterruptedException here.

		try
		{
			t1.join();
			t2.join();
			t3.join();
			t4.join();
		}
		catch (InterruptedException e)
		{
		}


		// Woo-hoo!  Now that we've successfully joined with the four annoying
		// kid threads, we know that all the annoying kids have stopped!  So
		// we'll print a final message and exit the program.

		System.out.println("All four annoying kids have stopped annoying us.");
	}
}
